/*
  Copyright (c) 1990-2005 Info-ZIP.  All rights reserved.

  See the accompanying file LICENSE, version 2004-May-22 or later
  (the contents of which are also included in zip.h) for terms of use.
  If, for some reason, both of these files are missing, the Info-ZIP license
  also may be found at:  ftp://ftp.info-zip.org/pub/infozip/license.html
*/
#ifndef __zipver_h   /* prevent multiple inclusions */
#define __zipver_h

#define ZIP_DLL_VERSION "2.31\0"
#define COMPANY_NAME "Info-ZIP\0"

#endif /* __zipver_h */
